@@ -76,13 +76,16 @@ def validate(identifier,what='doi'):
     if not identifier:
         return None
     if what=='doi':
-        if re.match('10\.\d{4,9}/[-._;()/:A-Z0-9]+',identifier,re.I):
+        if re.match(doi_pattern,identifier,re.I):
             if config.check_online_to_validate:
                 logger.info(f"Validating the possible DOI {identifier} via a query to dx.doi.org...")
                 result = bibtex_makers.doi2bib(identifier)
                 if result==-1:
                     logger.error(f"Some error occured during connection to dx.doi.org.")
                     return None
+                if isinstance(result,str) and result.strip()[0:5] == '@misc':
+                    logger.error(f"A valid bibTex entry was returned by dx.doi.org, but it starts with the tag \"@misc\". This might be the DOI of the journal and not the article itself.")
+                    return None
                 if result:
                     logger.info(f"The DOI {identifier} is validated by dx.doi.org. A bibtex entry was also created.")
                     return result
@@ -293,7 +296,7 @@ def find_possible_titles(path):
         title = pdftitle.get_title_from_file(path)
     except:
         title = ''
-    if len(title)>6:#This is to check that the title found is neither empty nor just few characters
+    if len(title.strip())>12:#This is to check that the title found is neither empty nor just few characters
         titles.append(title)  
         
     # (2)
@@ -302,11 +305,11 @@ def find_possible_titles(path):
     
     for key, value in info.items():
         if 'title' in key.lower():
-            if isinstance(value,str) and len(value)>6: #This is to check that the title found is neither empty nor just few characters
+            if isinstance(value,str) and len(value.strip())>12: #This is to check that the title found is neither empty nor just few characters
                 titles.append(value)         
     # (3)
     title = os.path.basename(path)
-    if len(title)>30:#This is to check that the title found is neither empty nor just few characters
+    if len(title.strip())>30:#This is to check that the title found is neither empty nor just few characters
         titles.append(title)
         
     return titles
@@ -355,58 +358,72 @@ def get_pdf_text(path,reader):
                 logger.error("An error occured while loading the document text with textract. The pdf version might be not supported.")
     return text
 
-def add_found_identifier_to_metadata(path,identifier):
-    """Given a pdf file identified by the input variable path, it adds a metadata to the pdf with name '/identifier'
-    and containing the content of the input variable identifier. This can be useful to make sure that the next time
-    this same pdf is analysed, the identifier is found more easily
+def add_found_identifier_to_metadata(target,identifier):
+    """Given a pdf file or a folder identified by the input variable target, it adds a metadata with name '/identifier'
+    and containing the content of the input variable identifier to either the pdf files (if target is the path of a single file) or
+    to all the pdf files in a folder (if target is the path of a folder). This can be useful to make sure that the next time
+    this same pdf is analysed, the identifier is found more easily.
+    It can also be useful when one want to reset to to '' the '/identifier' of all pdf files in a certain folder.
 
     Parameters
     ----------
-    path : string
-        a valid path to a pdf file
+    target : string
+        a valid path to a pdf file or a folder
     identifier : string
         a valid identifier, which will be stored in the pdf metadata with name '/identifier'
     Returns
     -------
     True if the the metadata was added succesfully, false otherwise
     """
-    logger.info(f"Trying to write the identifier \'{identifier}\' into the metadata of the file \'{path}\'...")
-    try:
-        file = open(path, 'rb') 
-    except (FileNotFoundError, IOError):
-        logger.error("File not found.")
-        return False
-    try:
-        pdf = PdfFileReader(path,strict=False)
-    except:
-        logger.error("It was not possible to open the file with PyPDF2. Is this a valid pdf file?")
-        return False
-    try:
-        writer = PdfFileWriter()
-        writer.appendPagesFromReader(pdf)
-        metadata = pdf.getDocumentInfo()
+    list_files = []
+    if  os.path.isdir(target): #if target is a folder, we populate the list list_files with all the pdf files contained in this folder
+        logger.info(f"Looking for pdf files in the folder {target}...")
+        pdf_files = [f for f in os.listdir(target) if f.endswith('.pdf')]
+        numb_files = len(pdf_files)
+        if len(pdf_files) == 0:
+            logger.error("No pdf files found in this folder.")
+            return None
+        logger.info(f"Found {numb_files} pdf files.")
+        if not(target.endswith(config.separator)): #Make sure the path ends with "\" or "/" (according to the OS)
+            target = target + config.separator
+        for f in pdf_files:
+            list_files.append(target + f)
+    else:
+        list_files = [target]
+
+    for f in list_files:
+        logger.info(f"Trying to write the identifier \'{identifier}\' into the metadata of the file \'{f}\'...")
         try:
-            writer.addMetadata(metadata)    #This instruction might generate an error if the pre-existing metadata are weird and are not
-                                            #correctly seen as strings (happens with old files). Therefore we use the try/except
-                                            #to ignore this possible problem
+            file = open(f, 'rb') 
+        except (FileNotFoundError, IOError):
+            logger.error("File not found.")
+            return False
+        try:
+            pdf = PdfFileReader(f,strict=False)
         except:
-            pass
-        key = '/identifier'
-        writer.addMetadata({
-            key: identifier
-        })
-    
-        fout = open(path, 'ab') 
-        writer.write(fout)
-    
-        file.close()
-        fout.close()
-        logger.info(f"The identifier \'{identifier}\' was added succesfully to the metadata of the file \'{path}\' with key \'{key}\'...")
-        return True
-    except Exception as e:
-        logger.error("Error from PyPDF2: " + str(e))
-        logger.error(f"An error occured while trying to write the identifier \'{identifier}\' into the metadata of the file \'{path}\'...")
-        return False
+            logger.error("It was not possible to open the file with PyPDF2. Is this a valid pdf file?")
+            return False
+        try:
+            writer = PdfFileWriter()
+            writer.appendPagesFromReader(pdf)
+            metadata = pdf.getDocumentInfo()
+            try:
+                writer.addMetadata(metadata)    #This instruction might generate an error if the pre-existing metadata are weird and are not
+            except:                             #correctly seen as strings (happens with old files). Therefore we use the try/except
+                pass                           #to ignore this possible problem
+            key = '/identifier'
+            writer.addMetadata({
+                key: identifier
+            })
+            fout = open(f, 'ab') 
+            writer.write(fout)
+            file.close()
+            fout.close()
+            logger.info(f"The identifier \'{identifier}\' was added succesfully to the metadata of the file \'{f}\' with key \'{key}\'...")
+        except Exception as e:
+            logger.error("Error from PyPDF2: " + str(e))
+            logger.error(f"An error occured while trying to write the identifier \'{identifier}\' into the metadata of the file \'{f}\'...")
+            return False
 
 
 ######## End first part ######## 
@@ -483,6 +500,7 @@ def find_identifier_by_googling_title(path, func_validate, numb_results=config.n
             return None, None, None
         else:
             logger.info(f"Found {len(titles)} possible title(s).")
+            titles.sort(key=len, reverse=True)
             for title in titles:
                 logger.info(f"Doing a google search for \"{title}\",")
                 logger.info(f"looking at the first {config.numb_results_google_search} results...")
@@ -550,6 +568,11 @@ def find_identifier_in_pdf_info(path,func_validate,keysToCheckFirst=[]):
     result : dictionary with identifier and other info (see above) 
     """
 
+
+    #This is a list of keys that will NOT be considered when looking for an identifier in the metadata. 
+    #Some of them are known to contain doi-like patterns but not the actual doi of the publication.
+    #For example '/wps-journaldoi' contains the DOI of the journal
+    KeysNotToUse = ['/wps-journaldoi'] 
     pdfinfo = get_pdf_info(path)
     identifier, desc, info = None, None, None
     if pdfinfo:
@@ -557,7 +580,7 @@ def find_identifier_in_pdf_info(path,func_validate,keysToCheckFirst=[]):
                                             #Some elements of keysToCheckFirst might be duplicated in 'Keys', but this is not a problem
                                             #because the corresponding element in the pdfinfo dictionary is eliminated after being checked
         for key in Keys:
-            if key in pdfinfo.keys():
+            if key in pdfinfo.keys() and key.lower() not in KeysNotToUse:
                 identifier,desc,info = find_identifier_in_text(pdfinfo[key],func_validate)
                 if identifier: 
                     logger.info(f"A valid {desc} was found in the document info labelled \'{key}\'.")
